/* 
   C support code for connection.pl 
   
   (C) Sandiway Fong, November 26th 2000. NEC Research Institute, Inc.

   Updated:

   Sandiway Fong, October 27th 2003. University of Arizona

*/
#include <stdio.h>
#include <SWI-Prolog.h>

/* Size of Wordnet 1.7 is 111223 synsets */
#define SIZE 111224

int mark[SIZE];

foreign_t 
reset_mark() 
{
  int i;

  for(i=0; i<SIZE; i++) 
    mark[i] = 0;
  return TRUE;
}


foreign_t 
set_mark(term_t i, term_t v) 
{
  int tmp1, tmp2;
  if ( PL_get_integer(i, &tmp1) && 
	   PL_get_integer(v, &tmp2)) {
      mark[tmp1] = tmp2;
  }
  return TRUE;
}


foreign_t 
get_mark(term_t i, term_t result) 
{
  int tmp1;
  
  if ( PL_get_integer(i, &tmp1)) {
       PL_unify_integer(result, mark[tmp1]);
  }
  return TRUE;
}


foreign_t  
count_marks(term_t result)
{
  int i;
  int total = 0;

  for(i=0; i<SIZE; i++)
    if( mark[i] > 0) total++;

  PL_unify_integer(result, total);

  return TRUE;
}


/* Path Tree */

#define PTSIZE 8000000		/* 8,000,000 * 4 bytes = 32Mb */
#define MARK  -2147483648   /* -2^31, old mar was 2^30 1073741824 */

int pt[PTSIZE];
int ptr  = 0;
int rptr = 0;			/* report ptr */
int zptr = 0;			/* zip ptr, used for expansion */
int sep  = 0;			/* last known location of the mark */
int sep2 = 0;			/* last known location of the previous mark */

foreign_t 
reset_pt()
{
  int i;

  pt[0] = MARK;

  for(i=1; i<PTSIZE; i++) 
    pt[i] = 0;

  ptr  = 1;
  sep  = 0;
  sep2 = 0;

  return TRUE;
}


foreign_t 
add_pt(term_t v, term_t p)
{
  int tmp1, tmp2; 
  
  if( ptr > (PTSIZE - 2)) {
    printf("\n Error: Path Tree overrun!\n");
    exit(2);
  }

  if ( PL_get_integer(v, &tmp1) && 
	   PL_get_integer(p, &tmp2)) {

	    pt[ptr++] = tmp2;
		pt[ptr++] = tmp1;
  }

  return TRUE;
}

foreign_t 
add_sep()
{
  sep2 = sep;
  sep = ptr;
  pt[ptr++] = MARK;
  return TRUE;
}


foreign_t 
zip_pt()			/* move back two marks */
{
  zptr = sep2;
  return TRUE;
}
  

foreign_t 
at_pt(term_t result)
{
  if( pt[++zptr] == MARK ) {
    PL_unify_integer(result, 0);
  }
  else {
	PL_unify_integer(result, pt[++zptr]); /* nb. two in front */
  }

  return TRUE;
}


foreign_t 
at2_pt(term_t result)
{
  PL_unify_integer(result, zptr);
  return TRUE;
}


foreign_t 
bk_pt( term_t i, term_t result)			/* back pointer value */
{
  int tmp1;

  if (PL_get_integer(i, &tmp1)){
	  PL_unify_integer(result, pt[tmp1]);
  }

  return TRUE;
}


foreign_t 
bk2_pt( term_t i, term_t result)		/* 2nd value is the next back pointer */
{ 
  int tmp1;

  if (PL_get_integer(i, &tmp1)){
	  PL_unify_integer(result, pt[tmp1-1]);
  }

  return TRUE;
}


foreign_t 
print_pt() 
{
  int i;

  for( i=0; i<ptr; i++)
    if( pt[i] == MARK ) {
      printf(" $ ");
    }
    else {
      printf(" %d",pt[i]);
    }

  printf(" (%d)\n",ptr);

  return TRUE;
}

install_t
install()
{
	PL_register_foreign("reset_mark", 0, reset_mark, 0);
	PL_register_foreign("set_mark", 2, set_mark, 0);
	PL_register_foreign("get_mark", 2, get_mark, 0);
	PL_register_foreign("count_marks", 1, count_marks, 0);
	PL_register_foreign("reset_pt", 0, reset_pt, 0);
	PL_register_foreign("add_pt", 2, add_pt, 0);
	PL_register_foreign("add_sep", 0, add_sep, 0);
	PL_register_foreign("zip_pt", 0, zip_pt, 0);
	PL_register_foreign("at_pt", 1, at_pt, 0);
	PL_register_foreign("at2_pt", 1, at2_pt, 0);
	PL_register_foreign("bk_pt", 2, bk_pt, 0);
	PL_register_foreign("bk2_pt", 2, bk2_pt, 0);
	PL_register_foreign("print_pt", 0, print_pt, 0);


}
      
	
     
